/*****************************************************************************/
/*** WMRVersieAlbertNL                                                     ***/
/*** Versie: 1.0                                                           ***/
/*** Datum: 21-10-2013                                                     ***/
/***                                                                       ***/
/*** Dit is de programma code voor De Nederlandse LMR Robot                ***/ 
/*** Wij maken robots!                                                     ***/
/***                                                                       ***/
/*** Deze versie is gebaseerd op WMRVersionVonFlo                          ***/
/*** Flo > Profiel: http://wirmachenroboter.com/user/19715                 ***/
/***                                                                       ***/
/*** Veel plezier ermee!                                                   ***/
/*** Albert > Profiel: http://letsmakerobots.com/user/21141                ***/
/*****************************************************************************/

// Praktische functies voor de servo aansturing komen hier vandaan:
#include <Servo.h> 

// Welke pin voor welke functie is geven we als volgt aan:
// Functie        Pin     Beschrijving
#define IR_SENSOR 2    // De Sharp IR Sensor
#define PWM_A     3    // Rijsnelheid A
#define SERVO     5    // Servo
#define BRAKE_B   8    // Rem B
#define BRAKE_A   9    // Rem A
#define PWM_B     11   // Rijsnelheid B
#define DIR_A     12   // Richting A
#define DIR_B     13   // Richting B


// Praktische instellingen:
#define FULL_SPEED        128             // Maximumsnelheid kan tot maximaal 255. Een hogere waarde dan deze is niet mogelijk.
#define TURN_SPEED        255             // Snel draaien.
#define LEFT              LOW             // Links draaien op LOW.
#define RIGHT             HIGH            // Rechts draaien op HIGH.
#define CLOSEST_DISTANCE  220             // Bij deze waarde voor de sensor zal de robot tot stilstand komen. Let op! Is de waarde KLEINER, dan stopt de robot eerder.
#define SERVO_RIGHT       55              // Tot welke hoek gaat de servo naar rechts. 
#define SERVO_LEFT        125             // Tot welke hoek gaat de servo naar links.

Servo SensorServo;  // Met dit element word de servo aangestuurd.

// Voor welke servo positie is de afstand tot het gevaar het kleinst?
int SeekingPositionWithClosestDanger()
{
  // Eerst tot stilstand komen.
  digitalWrite(BRAKE_A, HIGH); 
  digitalWrite(BRAKE_B, HIGH);
  int ServoPosition;
  int MinimumDistance = 0;
  int MinimumServoPosition = 0;
 
  // Van rechts (SERVO_RIGHT = 55) naar links (SERVO_LEFT=125) alle Servo Posities afgaan. Met ServoPositie++ word de waarde steeds met 1 verhoogd.
  for(ServoPosition = SERVO_RIGHT;ServoPosition <= SERVO_LEFT; ServoPosition++)
  {
    // Servo waarde instellen.
    SensorServo.write(ServoPosition);
    delay(10);
    // Is de actuele waarde van de sensor kleiner dan de minimale waarde?
    if(analogRead( IR_SENSOR ) > MinimumDistance )
    {
      // Ja: actuele waarde is het nieuwe minimum.
      MinimumDistance = analogRead( IR_SENSOR );
      // De Servo Positie vastleggen.
      MinimumServoPosition = ServoPosition; 
    }   
  }
  // De gevonden positie opnieuw instellen en de waarde daarvan teruggeven.
  SensorServo.write(MinimumServoPosition);
  return MinimumServoPosition;
}

byte ServoPosition = 90;
boolean TurnServo = RIGHT;
// Recht vooruit rijden en tegelijkertijd de sensor heen en weer bewegen.
void DriveForward()
{
  SensorServo.write( ServoPosition );
  // Beide motoren instellen op rechtdoor rijden.
  digitalWrite( DIR_A, HIGH );
  digitalWrite( DIR_B, HIGH );
  
  // Maximum ingestelde snelheid gebruiken.
  analogWrite( PWM_A, FULL_SPEED );
  analogWrite( PWM_B, FULL_SPEED );
  
  // Remmen los.
  digitalWrite( BRAKE_A, LOW );
  digitalWrite( BRAKE_B, LOW );
  
  // Draait de servo naar rechts?
  if( TurnServo == LEFT )
    ServoPosition = ServoPosition+1;  // Verder naar rechts draaien. Dat wil zeggen de waarde met 1 verhogen.
  if( TurnServo == RIGHT ) 
    ServoPosition = ServoPosition-1;  // Anders naar links draaien, de waarde met 1 verlagen.
    
  // Heeft de servo het maximum naar links bereikt?
  if( ServoPosition > SERVO_LEFT )
    TurnServo = RIGHT; // Dan naar rechts draaien.
  if( ServoPosition < SERVO_RIGHT )
    TurnServo = LEFT;  // Anders naar links draaien.
}

// Draaien, maar naar welke kant? LEFT voor links (tegen de klok in) RIGHT voor rechts.
void Turn( boolean Direction )
{
  // Remmen.
  digitalWrite( BRAKE_A, HIGH ); 
  digitalWrite( BRAKE_B, HIGH );
  delay( 500 );

  digitalWrite( DIR_A, Direction );    // Motor A in die "richting" draaien.
  digitalWrite( DIR_B, !Direction );   // Motor B in de tegenovergestelde "richting" draaien.
   
  // Snelheid voor het draaien instellen.
  analogWrite( PWM_A, TURN_SPEED );
  analogWrite( PWM_B, TURN_SPEED );
  
  // Remmen los.
  digitalWrite( BRAKE_A, LOW );
  digitalWrite( BRAKE_B, LOW );
  
  // Net zolang draaien totdat de sensor 10% verdere afstand meet.
  while( ( CLOSEST_DISTANCE * 1 ) < analogRead( IR_SENSOR ) )
  {
    delay( 50 );
  }
  // Stop.
  digitalWrite( BRAKE_A, HIGH ); 
  digitalWrite( BRAKE_B, HIGH );
  delay(1000);
}

// Voordat alles begint, word deze functie bij het opstarten eerst eenmalig uitgevoerd.
void setup( )
{
  //Motor A (rechts) initialiseren.
  pinMode( DIR_A, OUTPUT );    // De pin voor de richting van motor A als uitgang definiëren.
  pinMode( BRAKE_A, OUTPUT );  // De pin voor de rem van motor A als uitgang definiëren.
  //Motor B (links) initialisieren
  pinMode( DIR_B, OUTPUT );    // De pin voor de richting van motor B als uitgang definiëren.
  pinMode( BRAKE_B, OUTPUT );  // De pin voor de rem van motor B als uitgang definiëren.
  
  // Beide remmen in werking stellen. HIGH = remmen.
  digitalWrite( BRAKE_A, HIGH );
  digitalWrite( BRAKE_B, HIGH );

  // Servo initialiseren en op 90° instellen.
  SensorServo.attach( SERVO );
  SensorServo.write( 90 );
  delay( 500 );

  // Wachten tot er een object voor de sensor komt.
  while( CLOSEST_DISTANCE  > analogRead( IR_SENSOR ) )
  {
    // Zo lang moet er worden gewacht.
    delay( 100 );
  }
  
  // Beginnen!!
}

// Het eigenlijke programmaverloop. Nadat de setup( ) doorlopen is word de functie loop( ) eindeloos achter elkaar uitgevoerd.
void loop( )
{
  int DangerPosition;
  int Distance;
  
  Distance = analogRead( IR_SENSOR );    // Wat meet de sensor?
  if(Distance > CLOSEST_DISTANCE )       // Dichtbij genoeg? Is er een obstakel in de buurt, dan komt een hogere waarde terug, anders een kleinere.
  {
    DangerPosition = SeekingPositionWithClosestDanger();  // Nogmaals alles scannen om te weten waar het aankomende gevaar zich precies bevindt.
    
    // Is het gevaar aan de linkerkant?
    if(DangerPosition <= 90)
    {
      Turn( RIGHT );  // Draai naar rechts.
    }
    // Is het gevaar aan de rechterkant?
    if( DangerPosition > 90 )
    {
      Turn( LEFT );   // Draai naar links.
    }
  }
  DriveForward();  // Recht vooruit!
  delay( 10 );
}
